/*
	Written by TCH (tch@protonmail.com)
	$VER: 1.1.0
	Last modified: 2019.06.17.
	Public Domain
*/
#ifndef _SIGNALS_C
#define _SIGNALS_C

#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>
#include <time.h>

bool assign_signal(int signal, void (*callback)(int))
{
	struct sigaction act;

	act.sa_handler = callback;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
#ifdef __linux__
	act.sa_restorer = NULL;
#endif
	return sigaction(signal, &act, NULL) != 0;
}

bool signal_w_fsemaphore(pid_t proc, int sig, int32_t timeout, char *semaphore)
{
	struct timespec tmo, tmor;
	int res;
	bool no_timeout;

	if (proc == 0)
	{
		return false;
	}

	unlink(semaphore);
	kill(proc, sig);
	tmo.tv_sec = 0;
	no_timeout = timeout < 0;
	while ((access(semaphore, F_OK | R_OK) != 0) && (no_timeout || (timeout > 0)))
	{
		tmo.tv_nsec = 50000000;
		do
		{
			res = nanosleep(&tmo, &tmor);
			tmo.tv_nsec = tmor.tv_nsec;
		}
		while ((res == -1) && (errno == EINTR));
		timeout -= 50;
	}
	unlink(semaphore);
	return (!no_timeout) && (timeout < 1);
}

#endif